package edu.harvard.mgh.purcell.gplink.base_form;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * An extention of JPanel in which the user selects binary 
 * files (.bed, .bim, and .fam)
 * 
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class BinarySelectPanel extends JPanel {

	/**
	 * provides the form this panel will be shown in
	 */
	private Forms parent;
	
	/**
	 * text field that hold the .bed file name
	 */
	private GeneralUtil.UpdatingTextField BEDname, BIMname, FAMname;

	/**
	 * button to select the various files
	 */
	private GeneralUtil.pickAFileButton	selectBED, selectFAM, selectBIM;
	
	/**
	 * flags weather this is filled with a valid value or not
	 */
	public boolean validInput;

	/**
	 * check box that flags the use of --bfile
	 */
	private JCheckBox bfileCheck;
	/**
	 * combo box of valid --bfile arguments in the data.rootFile
	 */
	private JComboBox bfileCombo;
	
	/**
	 * An action listener that enabled the correct buttons
	 * based on the bfile checkbox
	 */
	private ActionListener checkAction = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
				selectBED.setEnabled(!bfileCheck.isSelected());
				selectFAM.setEnabled(!bfileCheck.isSelected());
				selectBIM.setEnabled(!bfileCheck.isSelected());
				bfileCombo.setEnabled(bfileCheck.isSelected());
				checkValidInput();
		}		
	};
	
	/**
	 * Get the command generated by this panel
	 * @param trim do we trim out the --<blah> or not? 
	 * if we do just return the file names
	 * @return a string containing the plink command for file inputs
	 */
	public String getCommand(boolean trim){
		String ans = "";
		
		if(validInput ==  false){//this should never happen!
			System.err.println("Invalid binary selection form");
			return ans;
		}
		
		if(trim == false){				//do we want to strip the commands off?
			if(bfileCheck.isSelected())	//is the --bfile flag selected?
				ans = "--bfile "
					+ GeneralUtil.quoteFilename(bfileCombo.getSelectedItem().toString());
			else
				ans = 
						" " + GeneralUtil.quoteFilename(BEDname.getText()) 
					 + " " + GeneralUtil.quoteFilename(BIMname.getText())
					 + " " + GeneralUtil.quoteFilename(FAMname.getText());
			}
		else
			if(bfileCheck.isSelected()){
				String tempRoot = bfileCombo.getSelectedItem().toString();
				ans = GeneralUtil.quoteFilename(tempRoot + ".bed")
					+ " " + GeneralUtil.quoteFilename(tempRoot + ".bim")
					+ " " + GeneralUtil.quoteFilename(tempRoot + ".fam");
			}
			else
				ans = 
					GeneralUtil.quoteFilename(BEDname.getText()) 
					+" " + GeneralUtil.quoteFilename(BIMname.getText())
					+" " + GeneralUtil.quoteFilename(FAMname.getText());
		
		return ans;
	}
	/**
	 * check to see if the binary input is valid
	 *
	 */
	private void checkValidInput(){
		//the form is valid if there is text in all three
		//...long form fields or if --bfile is flagged and the
		//...combo box has something non-trivial selected in it
		if((BEDname.getText().length() != 0
				&& BIMname.getText().length() !=0
				&& FAMname.getText().length() != 0)
			|| (bfileCheck.isSelected() == true
					&& bfileCombo.getSelectedItem()!= null
					&& !bfileCombo.getSelectedItem().equals("")))
			validInput = true;
		else
			validInput = false;
		
		//call the parent check to tell the buttons to
		//...enable if appropreate
		parent.isValidCommand();
	}
	
	private DocumentListener updatingOk_DL = new DocumentListener(){

		public void insertUpdate(DocumentEvent arg0) {
			checkValidInput();
		}

		public void removeUpdate(DocumentEvent arg0) {
			checkValidInput();
		}

		public void changedUpdate(DocumentEvent arg0) {
			checkValidInput();
		}
		
	};

	/**
	 * Constructer
	 * @param p Form that contains this panel.
	 * 
	 */
	public BinarySelectPanel(Forms p) {
		super();
		
		//keep track of what form this is going to be attached to
		parent = p;
		
		// set the sizes and content of the fileds that hold the
		// ...input files
		BEDname = new GeneralUtil.UpdatingTextField(10, updatingOk_DL);
		// don't let the user edit these field directly, they are editted
		// ...though a file chooser
		BEDname.setEditable(false);
		BIMname = new GeneralUtil.UpdatingTextField(10, updatingOk_DL);
		BIMname.setEditable(false);
		FAMname = new GeneralUtil.UpdatingTextField(10, updatingOk_DL);
		FAMname.setEditable(false);

		
		
		// set up buttons to trigger the previously mentioned file chooser
		selectBED = new GeneralUtil.pickAFileButton(".bed", "BED", BEDname, parent, false);
		//since we start assuming the user will want to use
		//...the short form we disenable this button
		//selectBED.setEnabled(false);
		selectBIM = new GeneralUtil.pickAFileButton(".bim", "BIM", BIMname, parent, false);
		//selectBIM.setEnabled(false);
		selectFAM = new GeneralUtil.pickAFileButton(".fam", "FAM", FAMname, parent, false);
		//selectFAM.setEnabled(false); 
		
		// add all the components to the pane
		JPanel longPanel = new JPanel();
		longPanel.setBorder(new EmptyBorder(5,5,5,5));
		longPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.ipady = 0;
		c.ipadx = 2;
		c.weighty = 1;
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL; 
		longPanel.add(new JLabel(".bed file:"), c);
		c.gridx = 1;
		c.weightx = 1;
		longPanel.add(BEDname, c);
		c.gridx=2;
		c.weightx = 0;
		longPanel.add(selectBED, c);
		c.gridy = 1;
		c.gridx = 0;
		longPanel.add(new JLabel(".bim file:"),c);
		c.gridx = 1;
		c.weightx = 1;
		longPanel.add(BIMname, c);
		c.gridx = 2;
		c.weightx = 0;
		longPanel.add(selectBIM, c);
		c.gridy=2;
		c.gridx = 0;
		longPanel.add(new JLabel(".fam file:"), c);
		c.gridx = 1;
		c.weightx = 1;
		longPanel.add(FAMname, c);
		c.gridx = 2;
		c.weightx = 0;
		longPanel.add(selectFAM, c);
		
		//create the quick panel
		JPanel bfilePanel =  new JPanel();
		bfilePanel.setBorder(new TitledBorder("Quick Fileset"));
		bfileCheck = new JCheckBox();
		bfileCheck.setSelected(true);
		bfileCheck.addActionListener(checkAction);
		populatebfileCombo();
		
		bfilePanel.setLayout(new BoxLayout(bfilePanel, BoxLayout.LINE_AXIS));
		bfilePanel.add(bfileCheck);
		bfilePanel.add(bfileCombo);
		bfilePanel.add(Box.createHorizontalGlue());
		
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.weightx = 1;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		add(bfilePanel, c);
		c.gridy = 1;
		add(longPanel, c);
	}

	/**
	 * Populate the --bfile combo box with valid strings based
	 * on the files in the project folder.
	 *
	 */
	private void populatebfileCombo() {
		//a list of all the files in the project folder that are
		//...binary inputs
		//File [] fileList = frame.data.getRoot()
		//	.listFiles(binInputFilter);
		
		Vector <String> fileList = parent.allFiles;
		
		//create a hash map to throw unique file roots into. We want to map
		//...the file root to the number of times it appears in the file list
		HashMap<String, Integer> allfileroots = new HashMap<String, Integer>();
		
		//go though the file list
		for(String file: fileList){
			if( file.endsWith(".bed") 
					|| file.endsWith(".bim")
					|| file.endsWith(".fam")){
				//chop off the extention
				//take the last three off
				String fileroot = GeneralUtil.fileName(file);
				fileroot = fileroot.substring(0, fileroot.length() - 4);
				//if the root exists
				if(allfileroots.containsKey(fileroot))
					//incrament the count
					allfileroots.put(fileroot, allfileroots.get(fileroot)+1);
				else
					//add it to the hash map
					allfileroots.put(fileroot, new Integer(1));
			}
		}
		
		//now we want to dump the hashmap into something that the
		//comboBox constructor will understand like a vector
		Vector <String>ans = new Vector <String> ();
		//check to make sure we found something
		if(allfileroots.isEmpty()){
			//add a blank string to it anyway so the combobox will
			//...get constructed correctly
			ans.add("");
			validInput = false;
		}		
		else{
			//iterate over the keys of the hashMap
			for( String key : allfileroots.keySet()){
				//if the root appears in the project folder 3 times
				if(allfileroots.get(key).equals(new Integer(3))){
					if(parent.myFrame.data.isflaggedRemote())
						ans.add(parent.myFrame.data.getRemoteFile()+"/" +key);
					else
					//add it to the vector
					ans.add(key);
				}
			}
			validInput = true;
		}
		//initalize the combobox
		bfileCombo = new JComboBox(ans);
	}
	
}
