package edu.harvard.mgh.purcell.gplink.mainGUI;

import javax.swing.*;

import edu.harvard.mgh.purcell.gplink.forms.CreatePlinkForm;
import edu.harvard.mgh.purcell.gplink.forms.RecodeForm;


import java.awt.Dimension;
import java.awt.event.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.Properties;

import edu.harvard.mgh.purcell.gplink.data.*;
import edu.harvard.mgh.purcell.gplink.forms.*;

/**
 * Provides the menu bar for GPLINK.
 * 
 * @author ktoddbrown
 * 
 */
@SuppressWarnings("serial")
public class GPLINKMenuBar extends JMenuBar {
	
	// Project that holds all the data. This lets us set the rootFolder
	// parameter.
	Project data;

	// panel that provides a view of the operations
	OpView operationsViewer;

	// frame that this is attached to
	MainFrame frame;

	//all the menus and their associated menu items
	JMenu fileMenu, analysisMenu, dataManMenu, statMenu, stratMenu, 
		assocMenu, ibdMenu;
	JMenuItem open, save;

	public JMenuItem export;

	JMenuItem exit;

	public JMenuItem options;

	JMenuItem update, about, project1, project2, project3, project4;
	JMenuItem commandLine, genFileset, mergeFiles, missingness, validate,
			mendelSumStats, hardySumStats, aFreqSumStats, hFreqSumStats, hPhaseSumStats,
			ibsStrat, clusterStrat, neighStrat,
			strat, basicAssoc, TDTAssoc, quantAssoc, glmAssoc, t2Assoc, fullAssoc,
			hapTDTAssoc, hapCCAssoc,
			stratAssoc, ibs, indHeter,
			importOp, addNonPlink;
	public JCheckBoxMenuItem browseOnly;
	/***************************************************************************
	 * Action Listeners for the menu items
	 **************************************************************************/

	/**
	 * general actionlistener that prints the menu item that activated it.
	 */
	ActionListener printListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			System.out.println("Menu item [" + event.getActionCommand()
					+ "] was pressed");
		}
	};
	
	ActionListener launchPrevious = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e){
			/*
			 * Close out the old project
			 */
			
			data.closeOldProject();

			/*
			 * Open the new project
			 */
			
			// Set the new folder in the project, this will automatically
			// ...update the rest of the project structure.
			data.setLocalRoot(new File(e.getActionCommand()));
			
			if(new File(data.getRoot(), data.REMOTE_FILE).exists() == false){
				//data.setRoot(new File(e.getActionCommand()));
				new ProjectDialog(frame, data.getRoot().getAbsolutePath());
			} else {
				Properties configRemote = new Properties();
				try {
					configRemote.load(new FileInputStream(new File(data.getRoot(), data.REMOTE_FILE)));
					new ProjectDialog(frame, data.getRoot().getAbsolutePath(),
							configRemote.getProperty(data.REMOTE_PROJECT_KEY),
							configRemote.getProperty(data.HOST_KEY), 
							configRemote.getProperty(data.USER_KEY),
							new Integer(configRemote.getProperty(data.PORT_KEY)));
					
				} catch (Exception e1) {
					new ProjectDialog(frame, data.getRoot().getAbsolutePath());
				}
			}
		}
	};


	ActionListener exportListener = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			data.writeOperations(true, false);
		}
	};
	
	ActionListener indHeterCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new IndvHeteroForm(frame);
		}
	};
	
	ActionListener clusterCall = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			new ClusteringForm(frame);
		}
	};
	
	ActionListener ibsDistCall = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			new IBSdistForm(frame);
		}
	};
	
	ActionListener aFreqCall = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			new AFreqForm(frame);
		}
		
	};
	
	ActionListener hFreqCall = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			new HFreqForm(frame);
		}
		
	};
	
	ActionListener hPhaseCall = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			new HPhaseForm(frame);
		}
		
	};

	ActionListener hardyCall = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			new HardyForm(frame);
		}
	};
	
	ActionListener mendelCall = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			new MendelForm(frame);
		}
	};

	/**
	 * call up a dialog to run a user inputed plink command
	 */
	ActionListener runPlink = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			new CreatePlinkForm(frame);
			/*if(data.isRemote())
				new PlinkExicute(frame, frame.data.getPlinkPrefix() + " " + 
						GeneralUtil.processFilename(frame.data.getPlinkPath())
						+ " --out {insert unique filename} !!use full path for all files!!");
			else
				new PlinkExicute(frame, frame.data.getPlinkPrefix() + " " + 
					GeneralUtil.processFilename(frame.data.getPlinkPath())
					+ " --out {insert unique filename}");
					*/
		}
	};

	/**
	 * Update the files that are displayed in the operations view panel
	 */
	ActionListener updateFiles = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			frame.updateViews(true);
		}
	};

	/**
	 * call a dialog to change the configurations options
	 */
	ActionListener setOptions = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			new ConfigDialog(frame);
		}
	};

	/**
	 * exit the program.
	 */
	ActionListener exitSystem = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			
			frame.properClosing.windowClosing(
					new WindowEvent(frame, WindowEvent.WINDOW_CLOSING));
		}
	};

	/**
	 * save the data in a .metafile in the open folder
	 */
	ActionListener saveData = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			data.writeProjectInfo(true, false, false);
		}
	};

	/**
	 * activate the Recode form.
	 */
	ActionListener recodeCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new RecodeForm(frame);
		}
	};
	
	ActionListener mergeCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new MergeForm(frame);
		}
	};
	
	ActionListener missingnessCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new MissingnessForm(frame);
		}
	};
	
	ActionListener validateCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new ValidateForm(frame);
		}
	};
	
	ActionListener basicAssocCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new BasicFullAssocForm(frame);
		}
	};
	
	ActionListener fullAssocCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new FullAssocForm(frame);
		}
	};
	
	ActionListener TDTCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new TDTForm(frame);
		}
	};
	
	ActionListener QuantCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new QuantAssocForm(frame);
		}
	};
	
	ActionListener glmCall = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			new GlmForm(frame);
		}
	};

	
	ActionListener T2Call = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new T2AssocForm(frame);
		}
	};
	
	ActionListener stratAssocCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new StratAssocForm(frame);
		}
	};
	
	ActionListener hapCCCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new HapCCAssocForm(frame);
		}
	};

	ActionListener hapTDTCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new HapTDTAssocForm(frame);
		}
	};

	ActionListener ibsCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new IBSForm(frame);
		}
	};

	ActionListener aboutCall = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent event){
			String message = "" +
			"gPLINK, version " + frame.VS + " \n" +
			"Released under GNU GPLv2 \n" +
			"Written by: Kathe Todd-Brown, Shaun Purcell \n" +
			"http://pngu.mgh.harvard.edu/purcell/plink/" +
			"\n";
			JOptionPane.showMessageDialog(frame, message);
		}
	};

	ActionListener neighCall = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new NeighForm(frame);
		}
	};
	
	/**
	 * sets the root folder in the main project.
	 */
	ActionListener getRootFolder = new ActionListener() {
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent event) {
			//data.closeOldProject();
			new ProjectDialog(frame);
		}
	};

	ActionListener setBrowse = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent event){
			/*********Load config***************/
			Properties config = new Properties();
			
			File configFile = new File(System.getProperty("user.dir")
					+ File.separator + data.GLOBAL_CONFIG_FILE);
			
			try{
				//System.out.println(configFile.getAbsolutePath());
				config.load(new FileInputStream(configFile));
			}catch(IOException e1){
				configFile = new File(System.getProperty("user.home")
						+ File.separator + data.GLOBAL_CONFIG_FILE);
				try{
					//System.out.println(configFile.getAbsolutePath());
					config.load(new FileInputStream(configFile));
				}catch(IOException e2){
					
				}
			}
			
			/**********set the config********/
			if(browseOnly.isSelected())
				config.setProperty(data.BROWSE_KEY, data.TRUE);
			else
				config.setProperty(data.BROWSE_KEY, data.FALSE);
			
						
			/*//if we havne't selected a root
			if(data.getRoot() == null){
				//set the 
				data.setBrowse(browseOnly.isSelected());
				//else if the browse button is selected
			}else if(browseOnly.isSelected()){
				//if we have the lock then releace it
				if(data.hasMetaLock() == true){
					data.unlockMeta();
				}
				data.setBrowse(true);
			}else{
				//if the lock is avalible then take it
				if(data.avalibleLock() == false){
					//then give the user an option to steal it
					int choice = JOptionPane.showConfirmDialog(null, 
							"Some other instance of gPLINK has the lock. " +
							"Would you like to steal the " +
							"lock?\n If you don't then you will be unable to add any new operations. ", 
							"Steal?", JOptionPane.YES_NO_OPTION);
					//if they choose to take the lock
					if(choice == JOptionPane.YES_OPTION){
						data.setBrowse(false);
						//lock the project
						data.lockMeta();						
					} else {
						data.setBrowse(true);
					}
				}else{
					data.setBrowse(false);
					data.lockMeta();
				}
			}
			*/
			
			/********set the data if it is non-null*****/
			if(data.getRoot() != null){
				if(browseOnly.isSelected()){
					if(data.hasMetaLock()){
						data.unlockMeta();
					} else {
					}
					data.setBrowse(true);
					setMenuLock(true);
				}else{
					data.setBrowse(false);
					if( data.avalibleLock()){
						data.lockMeta();
					}else {
						int choice = JOptionPane.showConfirmDialog(frame, 
								"Some other instance of gPLINK has the lock. " +
								"Would you like to steal the " +
								"lock? ", 
								"Steal?", JOptionPane.YES_NO_OPTION);
						if(choice == JOptionPane.CLOSED_OPTION
								|| choice == JOptionPane.NO_OPTION){
							data.setBrowse(true);
							setMenuLock(false);
						} else {
							data.lockMeta();
						}
					}
					data.setBrowse(false);
					setMenuLock(false);
				}
				//data.setBrowse(browseOnly.isSelected());
				//setMenuLock(data.hasMetaLock());
				options.setEnabled(true);
			}
			/*****Store the config************/
			try {
				config.store(new FileOutputStream(configFile), 
				"gPLINK global configuration");
			} catch (FileNotFoundException e) {
				new ErrorDialog(frame, "Error trying to write browse state" +
						" to the local configuration file.\n setBrowse@GPLINKMenuBar.java");
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to write browse state" +
				" to the local configuration file.\n setBrowse@GPLINKMenuBar.java");
			}
		}
	};
	
	public void setMenuLock(boolean hasLock){
		boolean show = hasLock && (!frame.data.getBrowse());
		//browseOnly.setSelected(!show);
		analysisMenu.setEnabled(show);
		//options.setEnabled(show);
		update.setEnabled(show);
		save.setEnabled(show);
	}
	
	@SuppressWarnings("static-access")
	public void setPreviousProjects(Properties config){
		if(config.isEmpty()
				|| config.getProperty(data.PROJECT1) == null
				|| config.getProperty(data.PROJECT2) == null
				|| config.getProperty(data.PROJECT3) == null
				|| config.getProperty(data.PROJECT4) == null)
			return;
		if(!config.getProperty(data.PROJECT1).equals("")){
			project1.setText(config.getProperty(data.PROJECT1));
			project1.setVisible(true);
		}
		if(!config.getProperty(data.PROJECT2).equals("")){
			project2.setText(config.getProperty(data.PROJECT2));
			project2.setVisible(true);
		}
		if(!config.getProperty(data.PROJECT3).equals("")){
			project3.setText(config.getProperty(data.PROJECT3));
			project3.setVisible(true);
		}
		if(!config.getProperty(data.PROJECT4).equals("")){
			project4.setText(config.getProperty(data.PROJECT4));
			project4.setVisible(true);
		}
	}
	
	public GPLINKMenuBar(MainFrame f) {
		data = f.data;
		frame = f;
		operationsViewer = frame.operationsView;
		
		/*****Project Menu ******/
		
		//create a new menu
		fileMenu = new JMenu("Project");
		//let the user be able to click on it
		fileMenu.setEnabled(true);
		//add it to the menubar
		add(fileMenu);
		
		//create a new button to go in the menu
		open = new JMenuItem("Set project folder");
		open.setEnabled(true);
		//action that happens when this button is clicked
		open.addActionListener(getRootFolder);
		//add it to the correct menu
		fileMenu.add(open);
		
		browseOnly = new JCheckBoxMenuItem("Only browse Projects");
		browseOnly.addActionListener(setBrowse);
		fileMenu.add(browseOnly);

		fileMenu.addSeparator();
		
		save = new JMenuItem("Save Project");
		save.setEnabled(false);
		save.addActionListener(saveData);
		fileMenu.add(save);
		
		export = new JMenuItem("Export Operations");
		export.setEnabled(false);
		export.addActionListener(exportListener);
		fileMenu.add(export);
		
		update = new JMenuItem("Rescan folder");
		update.setEnabled(false);
		update.addActionListener(updateFiles);
		fileMenu.add(update);

			
		options = new JMenuItem("Configuration");
		options.setEnabled(false);
		options.addActionListener(setOptions);
		fileMenu.add(options);

		fileMenu.addSeparator();

		project1 = new JMenuItem((String)null);
		project1.addActionListener(launchPrevious);
		project1.setVisible(false);
		fileMenu.add(project1);
		
		project2 = new JMenuItem((String)null);
		project2.addActionListener(launchPrevious);
		project2.setVisible(false);
		fileMenu.add(project2);
		
		project3 = new JMenuItem((String)null);
		project3.addActionListener(launchPrevious);
		project3.setVisible(false);
		fileMenu.add(project3);
		
		project4 = new JMenuItem((String) null);
		project4.addActionListener(launchPrevious);
		project4.setVisible(false);
		fileMenu.add(project4);
		
		fileMenu.addSeparator();
		
		exit = new JMenuItem("Exit");
		exit.setEnabled(true);
		exit.addActionListener(exitSystem);
		fileMenu.add(exit);
		
		/******PLINK menu******/
		
		analysisMenu = new JMenu("PLINK");
		analysisMenu.setEnabled(false);
		add(analysisMenu);
		
		dataManMenu = new JMenu("Data Management");
		dataManMenu.setEnabled(true);
		analysisMenu.add(dataManMenu);
		
		genFileset = new JMenuItem("Generate fileset");
		genFileset.setEnabled(true);
		genFileset.addActionListener(recodeCall);
		dataManMenu.add(genFileset);
		
		mergeFiles = new JMenuItem("Merge filesets");
		mergeFiles.addActionListener(mergeCall);
		dataManMenu.add(mergeFiles);
		
		statMenu = new JMenu("Summary Statistics");
		statMenu.setEnabled(true);
		analysisMenu.add(statMenu);
		
		validate = new JMenuItem("Validate fileset");
		validate.addActionListener(validateCall);
		statMenu.add(validate);

		missingness = new JMenuItem("Missingness");
		missingness.addActionListener(missingnessCall);
		statMenu.add(missingness);
				
		hardySumStats = new JMenuItem("Hardy Weinberg equilibrium");
		hardySumStats.addActionListener(hardyCall);
		statMenu.add(hardySumStats);

		mendelSumStats = new JMenuItem("Mendel error rates");
		mendelSumStats.addActionListener(mendelCall);
		statMenu.add(mendelSumStats);

		aFreqSumStats = new JMenuItem("Allele frequencies");
		aFreqSumStats.addActionListener(aFreqCall);
		statMenu.add(aFreqSumStats);

		hFreqSumStats = new JMenuItem("Haplotype frequencies");
		hFreqSumStats.addActionListener(hFreqCall);
		statMenu.add(hFreqSumStats);

		hPhaseSumStats = new JMenuItem("Haplotype phases");
		hPhaseSumStats.addActionListener(hPhaseCall);
		statMenu.add(hPhaseSumStats);

		stratMenu = new JMenu("Stratification");
		analysisMenu.add(stratMenu);
		
		ibsStrat = new JMenuItem("IBS distances");
		ibsStrat.addActionListener(ibsDistCall);
		stratMenu.add(ibsStrat);
		
		clusterStrat = new JMenuItem("Clustering");
		clusterStrat.addActionListener(clusterCall);
		stratMenu.add(clusterStrat);
				
		neighStrat = new JMenuItem("Nearest Neighbour");
		neighStrat.addActionListener(neighCall);
		stratMenu.add(neighStrat);
		
		assocMenu = new JMenu("Association");
		analysisMenu.add(assocMenu);
		
		basicAssoc = new JMenuItem("Allelic tests");
		basicAssoc.addActionListener(basicAssocCall);
		assocMenu.add(basicAssoc);
		
		fullAssoc = new JMenuItem("Genotypic tests (C/C)");
		fullAssoc.addActionListener(fullAssocCall);
		assocMenu.add(fullAssoc);
		
		TDTAssoc = new JMenuItem("Family-based tests (TDT)");
		TDTAssoc.addActionListener(TDTCall);
		assocMenu.add(TDTAssoc);
		
		stratAssoc = new JMenuItem("Stratified tests");
		stratAssoc.addActionListener(stratAssocCall);
		assocMenu.add(stratAssoc);
		
		quantAssoc = new JMenuItem("Quantitative trait interaction");
		quantAssoc.addActionListener(QuantCall);
		assocMenu.add(quantAssoc);
		
		glmAssoc = new JMenuItem("Regression tests");
		glmAssoc.addActionListener(glmCall);
		assocMenu.add(glmAssoc);

		t2Assoc = new JMenuItem("Set-based tests");
		t2Assoc.addActionListener(T2Call);
		assocMenu.add(t2Assoc);
		
		hapCCAssoc = new JMenuItem("Haplotype-based C/C tests");
		hapCCAssoc.addActionListener(hapCCCall);
		assocMenu.add(hapCCAssoc);

		hapTDTAssoc = new JMenuItem("Haplotype-based TDT tests");
		hapTDTAssoc.addActionListener(hapTDTCall);
		assocMenu.add(hapTDTAssoc);

		ibdMenu = new JMenu("IBD Estimation");
		analysisMenu.add(ibdMenu);
		
		indHeter = new JMenuItem("Individual heterozyogsity");
		indHeter.addActionListener(indHeterCall);
		ibdMenu.add(indHeter);
		
		ibs = new JMenuItem("Runs of homozygosity");
		ibs.addActionListener(ibsCall);
		ibdMenu.add(ibs);
		
		commandLine = new JMenuItem("Create PLINK command");
		commandLine.addActionListener(runPlink);
		analysisMenu.add(commandLine);
		
		importOp = new JMenuItem("Import Operation");
		importOp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				new ImportOpDialog(frame);
			}});
		analysisMenu.add(importOp);
		
		add(Box.createHorizontalGlue());
		
		addNonPlink = new JMenuItem("Add non-PLINK operation");
		addNonPlink.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				AddNonPlinkOpDialog temp = new AddNonPlinkOpDialog(frame);
				temp.pack();
				temp.setVisible(true);
			}
		});
		analysisMenu.add(addNonPlink);
		/*******About button on MenuBar*****/
		
		about = new JMenuItem("About");
		about.setMaximumSize(new Dimension(
				about.getPreferredSize().width, about.getMaximumSize().height));
		about.addActionListener(aboutCall);
		add(about);
		
		/********Populate menu items based on previous project******/
		reloadGPLINK_Config();
		
	}
	
	@SuppressWarnings({ "static-access", "static-access" })
	public void reloadGPLINK_Config(){
		/*********Load config***************/
		Properties config = new Properties();
		
		File configFile = new File(System.getProperty("user.dir")
				+ File.separator + data.GLOBAL_CONFIG_FILE);
		
		try{
			//System.out.println(configFile.getAbsolutePath());
			config.load(new FileInputStream(configFile));
		}catch(IOException e1){
			configFile = new File(System.getProperty("user.home")
					+ File.separator + data.GLOBAL_CONFIG_FILE);
			try{
				//System.out.println(configFile.getAbsolutePath());
				config.load(new FileInputStream(configFile));
			}catch(IOException e2){
				
			}
		}
		setPreviousProjects(config);
		
		/**
		 * Set the browse option
		 */
		if( config.get(data.BROWSE_KEY) == null
			|| config.get(data.BROWSE_KEY).equals(data.FALSE)){
				browseOnly.setSelected(false);
				data.setBrowse(false);
		}else{
			browseOnly.setSelected(true);
			data.setBrowse(true);
		}
		
		
	}

}
